/*******************************************\

  OpenGL code for Stroke Based Rendering
  Bump mapped oil paint strokes
  
  (c) John Collomosse 2005

  Created: JPC 22/6/05

  Extracted from "Empathic Painting" NPAR 2006 for 
  EG2011 NPR tutorial (Collomosse and Kyprianidis)

  Released under GPLv3
  http://www.gnu.org/licenses/gpl.html  

\*******************************************/
#include "cv.h"
#include "cxcore.h"
#include "highgui.h"
#include "sysparams.h"

typedef struct _renderparams {

		// dynamic
        float jaggedness;
		float undulation_freq;
		float undulation_amp;
		float dampen_bgstrokes;
		float bgcoverage;
		float colarousal;
		float colpleasant;

		//static 
		float maxstrokelen;
		float maxstrokeang;
        
} RENDERPARAMS;

typedef struct _spline {
	
	int numPoints;
	float* lengths;
	float totallength;
	CvPoint2D32f *pts;
} SPLINE;

typedef struct _stroke {

	int id;
	SPLINE* spline;
	int		red,green,blue;
	float   thickness;
	float	depth;
	float ERM; //ephemeral random number - seeded at creation, but persistent over renderings
	int isBackground;
	float bgcoverage;
	struct _stroke* prev;
	struct _stroke* next;

} STROKE;


typedef struct _qsortelem {
        double sortidx;
        STROKE* ptr;
} QSELEM;

extern int IMAGE_HEIGHT;
extern int IMAGE_WIDTH;
extern void	addStroke(CvPoint2D32f*, int, int, int, int, float, float, STROKE*, int,float);
extern void buildcumarray(double*, IplImage*);
extern double grand(void);
extern double round (double);
extern CvPoint pickpoint(double*,int ,int);
extern int countStrokes (void);
extern void buildTestPainting(void);
extern int comp(const void*, const void*);
extern void sortPaintingList(void);

extern CvRNG		rng_state;			
extern RENDERPARAMS renderparams;

